<?php

namespace FiloBlu\OrderFlowRectifier\Model;

use FiloBlu\OrderFlowRectifier\Helper\Data;

class Processor extends \Magento\Framework\Model\AbstractModel
{

    const MAX_ORDER_TO_PROCESS = 10;
    const ENABLED_RECTIFIERS_CONFIG_PATH = 'order_flow_rectifier/general/enabled_rectifiers';
    const LOG_FILE_NAME = '/var/log/filoblu_order_flow_rectifier.log';

    /**
     * @var RectifierPoolFactory
     */
    protected $rectifierPoolFactory;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $config;

    /**
     * @var Data
     */
    protected $helper;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        RectifierPoolFactory $rectifierPoolFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $config,
        Data $helper,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = [])
    {
        parent::__construct(
            $context, $registry, $resource, $resourceCollection, $data);

        $this->rectifierPoolFactory = $rectifierPoolFactory;
        $this->config = $config;
        $this->helper = $helper;
    }

    public function process() {
        $rectifiers = $this->rectifierPoolFactory->create()->getList();
        $enabledRectifiers = $this->config->getValue(self::ENABLED_RECTIFIERS_CONFIG_PATH);
        $enabledRectifiers = array_map('trim', explode(',', $enabledRectifiers));
        foreach ($rectifiers as $code => $rectifier) {
            if (!in_array($code, $enabledRectifiers)) {
                continue;
            }
            $ordersToProcess = $rectifier->loadOrders();
            if (count($ordersToProcess) > self::MAX_ORDER_TO_PROCESS) {
                $this->helper->log(self::LOG_FILE_NAME, 'Running rectifier "' . $rectifier->getTitle() . '" on ' . self::MAX_ORDER_TO_PROCESS . ' of ' . count($ordersToProcess) . ' orders');
            } else {
                $this->helper->log(self::LOG_FILE_NAME, 'Running rectifier "' . $rectifier->getTitle() . '" on ' . count($ordersToProcess) . ' orders');
            }
            $processedCount = 0;
            foreach ($ordersToProcess as $order) {
                if ($processedCount > self::MAX_ORDER_TO_PROCESS) {
                    break;
                }
                $this->helper->log(self::LOG_FILE_NAME, 'Processing order ' . $order->increment_id);
                $success = $rectifier->executeSingle($order);
                if ($success) {
                    $this->helper->log(self::LOG_FILE_NAME, 'Done');
                } else {
                    $this->helper->log(self::LOG_FILE_NAME, 'Not done');
                }
                $processedCount++;
            }
        }
        $this->helper->log(self::LOG_FILE_NAME, 'END RECTIFY PROCESS');
    }

}